@@ -7,7 +7,10 @@ from ctypes import POINTER
 from ctypes import pointer
 from ctypes import Structure
 
+import psutil
 import dask.array as da
+from dask import config as da_conf
+from dask import utils as da_utils
 import numpy as np
 import xarray as xr
 from py import path
@@ -21,6 +24,7 @@ from parcels.tools.converters import GeographicPolar
 from parcels.tools.converters import TimeConverter
 from parcels.tools.converters import UnitConverter
 from parcels.tools.converters import unitconverters_map
+from parcels.tools.converters import convert_xarray_time_units
 from parcels.tools.error import FieldOutOfBoundError
 from parcels.tools.error import FieldOutOfBoundSurfaceError
 from parcels.tools.error import FieldSamplingError
@@ -176,7 +180,7 @@ class Field(object):
 
     @classmethod
     def get_dim_filenames(cls, filenames, dim):
-        if isinstance(filenames, str) or not isinstance(filenames, collections.Iterable):
+        if isinstance(filenames, str) or not isinstance(filenames, collections.abc.Iterable):
             return [filenames]
         elif isinstance(filenames, dict):
             assert dim in filenames.keys(), \
@@ -221,8 +225,6 @@ class Field(object):
                that case Parcels deals with a better memory management during particle set execution.
                deferred_load=False is however sometimes necessary for plotting the fields.
         :param field_chunksize: size of the chunks in dask loading
-        :param netcdf_engine: engine to use for netcdf reading in xarray. Default is 'netcdf',
-               but in cases where this doesn't work, setting netcdf_engine='scipy' could help
         """
         # Ensure the timestamps array is compatible with the user-provided datafiles.
         if timestamps is not None:
@@ -230,35 +232,30 @@ class Field(object):
                 assert len(filenames) == len(timestamps), 'Outer dimension of timestamps should correspond to number of files.'
             elif isinstance(filenames, dict):
                 for k in filenames.keys():
-                    assert(len(filenames[k]) == len(timestamps)), 'Outer dimension of timestamps should correspond to number of files.'
+                    if k not in ['lat','lon','depth','time']:
+                        assert(len(filenames[k]) == len(timestamps)), 'Outer dimension of timestamps should correspond to number of files.'
             else:
                 raise TypeError("Filenames type is inconsistent with manual timestamp provision."
                                 + "Should be dict or list")
 
-        if isinstance(variable, xr.core.dataarray.DataArray):
-            lonlat_filename = variable
-            depth_filename = variable
-            data_filenames = variable
-            netcdf_engine = 'xarray'
-        else:
-            if isinstance(variable, str):  # for backward compatibility with Parcels < 2.0.0
-                variable = (variable, variable)
-            assert len(variable) == 2, 'The variable tuple must have length 2. Use FieldSet.from_netcdf() for multiple variables'
+        if isinstance(variable, str):  # for backward compatibility with Parcels < 2.0.0
+            variable = (variable, variable)
+        assert len(variable) == 2, 'The variable tuple must have length 2. Use FieldSet.from_netcdf() for multiple variables'
 
-            data_filenames = cls.get_dim_filenames(filenames, 'data')
-            lonlat_filename = cls.get_dim_filenames(filenames, 'lon')
-            if isinstance(filenames, dict):
-                assert len(lonlat_filename) == 1
-            if lonlat_filename != cls.get_dim_filenames(filenames, 'lat'):
-                raise NotImplementedError('longitude and latitude dimensions are currently processed together from one single file')
-            lonlat_filename = lonlat_filename[0]
-            if 'depth' in dimensions:
-                depth_filename = cls.get_dim_filenames(filenames, 'depth')
-                if isinstance(filenames, dict) and len(depth_filename) != 1:
-                    raise NotImplementedError('Vertically adaptive meshes not implemented for from_netcdf()')
-                depth_filename = depth_filename[0]
+        data_filenames = cls.get_dim_filenames(filenames, 'data')
+        lonlat_filename = cls.get_dim_filenames(filenames, 'lon')
+        if isinstance(filenames, dict):
+            assert len(lonlat_filename) == 1
+        if lonlat_filename != cls.get_dim_filenames(filenames, 'lat'):
+            raise NotImplementedError('longitude and latitude dimensions are currently processed together from one single file')
+        lonlat_filename = lonlat_filename[0]
+        if 'depth' in dimensions:
+            depth_filename = cls.get_dim_filenames(filenames, 'depth')
+            if isinstance(filenames, dict) and len(depth_filename) != 1:
+                raise NotImplementedError('Vertically adaptive meshes not implemented for from_netcdf()')
+            depth_filename = depth_filename[0]
 
-            netcdf_engine = kwargs.pop('netcdf_engine', 'netcdf4')
+        netcdf_engine = kwargs.pop('netcdf_engine', 'netcdf4')
 
         indices = {} if indices is None else indices.copy()
         for ind in indices.values():
@@ -276,7 +273,7 @@ class Field(object):
             else:
                 raise RuntimeError('interp_method is a dictionary but %s is not in it' % variable[0])
 
-        with NetcdfFileBuffer(lonlat_filename, dimensions, indices, netcdf_engine) as filebuffer:
+        with NetcdfFileBuffer(lonlat_filename, dimensions, indices, netcdf_engine, field_chunksize=False) as filebuffer:
             lon, lat = filebuffer.read_lonlat
             indices = filebuffer.indices
             # Check if parcels_mesh has been explicitly set in file
@@ -284,7 +281,7 @@ class Field(object):
                 mesh = filebuffer.dataset.attrs['parcels_mesh']
 
         if 'depth' in dimensions:
-            with NetcdfFileBuffer(depth_filename, dimensions, indices, netcdf_engine, interp_method=interp_method) as filebuffer:
+            with NetcdfFileBuffer(depth_filename, dimensions, indices, netcdf_engine, interp_method=interp_method, field_chunksize=False) as filebuffer:
                 filebuffer.name = filebuffer.parse_name(variable[1])
                 depth = filebuffer.read_depth
                 data_full_zdim = filebuffer.data_full_zdim
@@ -308,17 +305,12 @@ class Field(object):
                         dataFiles.append(f)
                 timeslices = np.array([stamp for file in timestamps for stamp in file])
                 time = timeslices
-            elif netcdf_engine == 'xarray':
-                with NetcdfFileBuffer(data_filenames, dimensions, indices, netcdf_engine) as filebuffer:
-                    time = filebuffer.time
-                    timeslices = time if isinstance(time, (list, np.ndarray)) else [time]
-                    dataFiles = data_filenames if isinstance(data_filenames, (list, np.ndarray)) else [data_filenames] * len(time)
             else:
                 timeslices = []
                 dataFiles = []
                 for fname in data_filenames:
                     with NetcdfFileBuffer(fname, dimensions, indices, netcdf_engine,
-                                          field_chunksize=field_chunksize) as filebuffer:
+                                          field_chunksize=False) as filebuffer:
                         ftime = filebuffer.time
                         timeslices.append(ftime)
                         dataFiles.append([fname] * len(ftime))
@@ -352,13 +344,10 @@ class Field(object):
             for tslice, fname in zip(grid.timeslices, data_filenames):
                 with NetcdfFileBuffer(fname, dimensions, indices, netcdf_engine,
                                       interp_method=interp_method, data_full_zdim=data_full_zdim,
-                                      field_chunksize=field_chunksize) as filebuffer:
+                                      field_chunksize=False) as filebuffer:
                     # If Field.from_netcdf is called directly, it may not have a 'data' dimension
                     # In that case, assume that 'name' is the data dimension
-                    if netcdf_engine == 'xarray':
-                        tslice = [tslice]
-                    else:
-                        filebuffer.name = filebuffer.parse_name(variable[1])
+                    filebuffer.name = filebuffer.parse_name(variable[1])
                     buffer_data = filebuffer.data
                     if len(buffer_data.shape) == 2:
                         data_list.append(buffer_data.reshape(sum(((len(tslice), 1), buffer_data.shape), ())))
@@ -369,6 +358,8 @@ class Field(object):
                             data_list.append(buffer_data.reshape(sum(((len(tslice), 1), buffer_data.shape[1:]), ())))
                     else:
                         data_list.append(buffer_data)
+                    if type(tslice) not in [list, np.ndarray]:
+                        tslice = [tslice]
                 ti += len(tslice)
             lib = np if isinstance(data_list[0], np.ndarray) else da
             data = lib.concatenate(data_list, axis=0)
@@ -386,10 +377,45 @@ class Field(object):
         kwargs['netcdf_engine'] = netcdf_engine
         kwargs['field_chunksize'] = field_chunksize
 
-        variable = kwargs['var_name'] if netcdf_engine == 'xarray' else variable
         return cls(variable, data, grid=grid, timestamps=timestamps,
                    allow_time_extrapolation=allow_time_extrapolation, interp_method=interp_method, **kwargs)
 
+    @classmethod
+    def from_xarray(cls, da, name, dimensions, mesh='spherical', allow_time_extrapolation=None,
+                    time_periodic=False, **kwargs):
+        """Create field from xarray Variable
+
+        :param da: Xarray DataArray
+        :param name: Name of the Field
+        :param dimensions: Dictionary mapping variable names for the relevant dimensions in the DataArray
+        :param mesh: String indicating the type of mesh coordinates and
+               units used during velocity interpolation:
+
+               1. spherical (default): Lat and lon in degree, with a
+                  correction for zonal velocity U near the poles.
+               2. flat: No conversion, lat/lon are assumed to be in m.
+        :param allow_time_extrapolation: boolean whether to allow for extrapolation in time
+               (i.e. beyond the last available time snapshot)
+               Default is False if dimensions includes time, else True
+        :param time_periodic: boolean whether to loop periodically over the time component of the FieldSet
+               This flag overrides the allow_time_interpolation and sets it to False
+        """
+
+        data = da.values
+        interp_method = kwargs.pop('interp_method', 'linear')
+
+        time = da[dimensions['time']].values if 'time' in dimensions else np.array([0])
+        depth = da[dimensions['depth']].values if 'depth' in dimensions else np.array([0])
+        lon = da[dimensions['lon']].values
+        lat = da[dimensions['lat']].values
+
+        time_origin = TimeConverter(time[0])
+        time = time_origin.reltime(time)
+
+        grid = Grid.create_grid(lon, lat, depth, time, time_origin=time_origin, mesh=mesh)
+        return cls(name, data, grid=grid, allow_time_extrapolation=allow_time_extrapolation,
+                   interp_method=interp_method, **kwargs)
+
     def reshape(self, data, transpose=False):
 
         # Ensure that field data is the right data type
@@ -812,6 +838,10 @@ class Field(object):
             # If given time > last known field time, use
             # the last field frame without interpolation
             return (len(self.grid.time) - 1, 0)
+        elif np.logical_not(time_index).all():
+            # If given time < any time in the field, use
+            # the first field frame without interpolation
+            return (0, 0)
         else:
             return (time_index.argmin() - 1 if time_index.any() else 0, 0)
 
@@ -1078,13 +1108,18 @@ class Field(object):
         timestamp = self.timestamps
         if timestamp is not None:
             summedlen = np.cumsum([len(ls) for ls in self.timestamps])
-            timestamp = self.timestamps[np.where(g.ti + tindex < summedlen)[0][0]]
+            if g.ti + tindex >= summedlen[-1]:
+                ti = g.ti + tindex - summedlen[-1]
+            else:
+                ti = g.ti + tindex
+            timestamp = self.timestamps[np.where(ti < summedlen)[0][0]]
 
         filebuffer = NetcdfFileBuffer(self.dataFiles[g.ti + tindex], self.dimensions, self.indices,
                                       self.netcdf_engine, timestamp=timestamp,
                                       interp_method=self.interp_method,
                                       data_full_zdim=self.data_full_zdim,
-                                      field_chunksize=self.field_chunksize)
+                                      field_chunksize=self.field_chunksize,
+                                      rechunk_callback_fields=self.chunk_setup)
         filebuffer.__enter__()
         time_data = filebuffer.time
         time_data = g.time_origin.reltime(time_data)
@@ -1508,7 +1543,7 @@ class NestedField(list):
 class NetcdfFileBuffer(object):
     """ Class that encapsulates and manages deferred access to file data. """
     def __init__(self, filename, dimensions, indices, netcdf_engine, timestamp=None,
-                 interp_method='linear', data_full_zdim=None, field_chunksize='auto'):
+                 interp_method='linear', data_full_zdim=None, field_chunksize='auto', rechunk_callback_fields=None):
         self.filename = filename
         self.dimensions = dimensions  # Dict with dimension keys for file data
         self.indices = indices
@@ -1519,18 +1554,21 @@ class NetcdfFileBuffer(object):
         self.interp_method = interp_method
         self.data_full_zdim = data_full_zdim
         self.field_chunksize = field_chunksize
+        self.chunk_mapping = None
+        self.rechunk_callback_fields = rechunk_callback_fields
 
     def __enter__(self):
         if self.netcdf_engine == 'xarray':
             self.dataset = self.filename
             return self
+        init_chunk_dict = self._get_initial_chunk_dictionary()
         try:
-            self.dataset = xr.open_dataset(str(self.filename), decode_cf=True, engine=self.netcdf_engine)
+            self.dataset = xr.open_dataset(str(self.filename), decode_cf=True, engine=self.netcdf_engine, chunks=init_chunk_dict)
             self.dataset['decoded'] = True
         except:
             logger.warning_once("File %s could not be decoded properly by xarray (version %s).\n         It will be opened with no decoding. Filling values might be wrongly parsed."
                                 % (self.filename, xr.__version__))
-            self.dataset = xr.open_dataset(str(self.filename), decode_cf=False, engine=self.netcdf_engine)
+            self.dataset = xr.open_dataset(str(self.filename), decode_cf=False, engine=self.netcdf_engine, chunks=init_chunk_dict)
             self.dataset['decoded'] = False
         for inds in self.indices.values():
             if type(inds) not in [list, range]:
@@ -1542,6 +1580,71 @@ class NetcdfFileBuffer(object):
             pass
         else:
             self.dataset.close()
+            self.dataset = None
+
+    def _get_initial_chunk_dictionary(self):
+        # ==== check-opening requested dataset to access metadata ==== #
+        try:
+            self.dataset = xr.open_dataset(str(self.filename), decode_cf=True, engine=self.netcdf_engine, chunks={})
+            self.dataset['decoded'] = True
+        except:
+            logger.warning_once("File %s could not be decoded properly by xarray (version %s).\n         It will be opened with no decoding. Filling values might be wrongly parsed."
+                                % (self.filename, xr.__version__))
+            self.dataset = xr.open_dataset(str(self.filename), decode_cf=False, engine=self.netcdf_engine, chunks={})
+            self.dataset['decoded'] = False
+        # ==== self.dataset temporarily available ==== #
+        init_chunk_dict = {}
+        if isinstance(self.field_chunksize, dict):
+            init_chunk_dict = self.field_chunksize
+        elif isinstance(self.field_chunksize, tuple) and (len(self.dimensions) == self.field_chunksize):
+            chunk_index = 0
+            if self._is_dimension_available('time'):
+                init_chunk_dict[self.dimensions['time']] = self.field_chunksize[chunk_index] if len(self.dimensions) > 2 else 1
+                chunk_index += 1 if len(self.dimensions) > 2 else 0
+            else:
+                logger.warning_once(self._netcdf_DimNotFound_warning_message('time'))
+            if self._is_dimension_available('depth'):
+                init_chunk_dict[self.dimensions['depth']] = self.field_chunksize[chunk_index] if len(self.dimensions) > 3 else 1
+                chunk_index += 1 if len(self.dimensions) > 3 else 0
+            else:
+                logger.warning_once(self._netcdf_DimNotFound_warning_message('depth'))
+            if self._is_dimension_available('lat'):
+                init_chunk_dict[self.dimensions['lat']] = self.field_chunksize[chunk_index]
+                chunk_index += 1
+            else:
+                logger.warning_once(self._netcdf_DimNotFound_warning_message('lat'))
+            if self._is_dimension_available('lom'):
+                init_chunk_dict[self.dimensions['lon']] = self.field_chunksize[chunk_index]
+            else:
+                logger.warning_once(self._netcdf_DimNotFound_warning_message('lon'))
+        elif self.field_chunksize == 'auto':
+            av_mem = psutil.virtual_memory().available
+            chunk_cap = av_mem * (1/8) * (1/3)
+            if 'array.chunk-size' in da_conf.config.keys():
+                chunk_cap = da_utils.parse_bytes(da_conf.config.get('array.chunk-size'))
+            else:
+                logger.warning_once("Unable to locate chunking hints from dask, thus estimating the max. chunk size heuristically. Please consider defining the 'chunk-size' for 'array' in your local dask configuration file (see https://docs.dask.org).")
+            if self._is_dimension_available('lat') and self._is_dimension_available('lon;'):
+                pDim = int(math.floor(math.sqrt(chunk_cap/np.dtype(np.float64).itemsize)))
+                init_chunk_dict[self.dimensions['lat']] = pDim
+                init_chunk_dict[self.dimensions['lon']] = pDim
+            if self._is_dimension_available('time'):
+                init_chunk_dict[self.dimensions['time']] = 1
+            if self._is_dimension_available('depth'):
+                init_chunk_dict[self.dimensions['depth']] = 1
+        # ==== closing check-opened requested dataset ==== #
+        self.dataset.close()
+        self.dataset = None
+        # ==== self.dataset not available ==== #
+        return init_chunk_dict
+
+    def _is_dimension_available(self, dimension_name):
+        if self.dimensions is None or self.dataset is None:
+            return False
+        return (dimension_name in self.dimensions and self.dimensions[dimension_name] in self.dataset.dims)
+
+    def _netcdf_DimNotFound_warning_message(self, dimension_name):
+        return "Did not find {} in NetCDF dims. Please specifiy field_chunksize as dictionary for NetCDF dimension names, e.g.\n field_chunksize=\{ '{}': <number>, ... \}.".format(self.dimensions[dimension_name], self.dimensions[dimension_name])
 
     def parse_name(self, name):
         if isinstance(name, list):
@@ -1610,10 +1713,25 @@ class NetcdfFileBuffer(object):
 
     @property
     def data(self):
-        if self.netcdf_engine == 'xarray':
-            data = self.dataset
-        else:
-            data = self.dataset[self.name]
+        return self.data_access()
+
+    def data_access(self):
+        if self.chunk_mapping is None and self.field_chunksize not in ['auto', False]:
+            self.chunk_mapping = {}
+            if(isinstance(self.field_chunksize, tuple)):
+                for i in range(len(self.field_chunksize)):
+                    self.chunk_mapping[i] = self.field_chunksize[i]
+            else:
+                dim_max_index = len(self.dimensions)-1
+                dim_index = 0
+                for dim_name in self.dimensions:
+                    coord_id = dim_max_index - dim_index
+                    if self.dimensions[dim_name] in self.field_chunksize:
+                        self.chunk_mapping[coord_id] = self.field_chunksize[self.dimensions[dim_name]]
+                    else:
+                        self.chunk_mapping[coord_id] = 1
+                    dim_index += 1
+        data = self.dataset[self.name]
         ti = range(data.shape[0]) if self.ti is None else self.ti
         if len(data.shape) == 2:
             data = data[self.indices['lat'], self.indices['lon']]
@@ -1661,14 +1779,24 @@ class NetcdfFileBuffer(object):
             else:
                 data = data[ti, self.indices['depth'], self.indices['lat'], self.indices['lon']]
 
+        if isinstance(data, xr.DataArray):
+            data = data.data
         if self.field_chunksize is False:
             data = np.array(data)
         else:
             if isinstance(data, da.core.Array):
                 if self.field_chunksize == 'auto' and data.shape[-2:] == data.chunksize[-2:]:
-                    data = np.array(data)
-                elif self.field_chunksize != 'auto':
+                    pass
+                elif self.field_chunksize == 'auto' and self.rechunk_callback_fields is not None:
                     data = data.rechunk(self.field_chunksize)
+                    self.chunk_mapping = {}
+                    chunkIndex = 0
+                    for chunkDim in data.numblocks[1:]:
+                        self.chunk_mapping[chunkIndex] = chunkDim
+                        chunkIndex += 1
+                    self.rechunk_callback_fields()
+                elif self.field_chunksize != 'auto':
+                    data = data.rechunk(self.chunk_mapping)
             else:
                 da_data = da.from_array(data, chunks=self.field_chunksize)
                 if self.field_chunksize == 'auto' and da_data.shape[-2:] == da_data.chunksize[-2:]:
@@ -1687,21 +1815,8 @@ class NetcdfFileBuffer(object):
             return np.array([None])
 
         time_da = self.dataset[self.dimensions['time']]
-        if self.netcdf_engine != 'xarray' and (self.dataset['decoded'] and 'Unit' not in time_da.attrs):
-            time = np.array([time_da]) if len(time_da.shape) == 0 else np.array(time_da)
-        else:
-            if 'units' not in time_da.attrs and 'Unit' in time_da.attrs:
-                time_da.attrs['units'] = time_da.attrs['Unit']
-            ds = xr.Dataset({self.dimensions['time']: time_da})
-            try:
-                ds = xr.decode_cf(ds)
-            except ValueError:
-                raise RuntimeError('Xarray could not convert the calendar. Try using the timestamps '
-                                   'keyword in the construction of your Field %s. See also the tutorial '
-                                   'at https://nbviewer.jupyter.org/github/OceanParcels/parcels/blob/master'
-                                   '/parcels/examples/tutorial_timestamps.ipynb' % self.filename)
-            da = ds[self.dimensions['time']]
-            time = np.array([da]) if len(da.shape) == 0 else np.array(da)
+        convert_xarray_time_units(time_da, self.dimensions['time'])
+        time = np.array([time_da[self.dimensions['time']]]) if len(time_da.shape) == 0 else np.array(time_da[self.dimensions['time']])
         if isinstance(time[0], datetime.datetime):
             raise NotImplementedError('Parcels currently only parses dates ranging from 1678 AD to 2262 AD, which are stored by xarray as np.datetime64. If you need a wider date range, please open an Issue on the parcels github page.')
         return time
