@@ -1,9 +1,13 @@
-from parcels.compiler import get_cache_dir, GNUCompiler
-from parcels.tools.loggers import logger
-from os import path
-import numpy.ctypeslib as npct
-from ctypes import c_int, c_float
 import uuid
+from ctypes import c_float
+from ctypes import c_int
+from os import path
+
+import numpy.ctypeslib as npct
+
+from parcels.compiler import get_cache_dir
+from parcels.compiler import GNUCompiler
+from parcels.tools.loggers import logger
 
 
 __all__ = ['seed', 'random', 'uniform', 'randint', 'normalvariate', 'expovariate']
